﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Diagnostics;
using System.ComponentModel;

namespace BSFControls.ExtendedControls
{
    class KATButton : Control
    {
        public bool AutoSizeImage { get; set; }
        private int _cornerRadius = 5;
        private Image _disabledBackgroundImage;
        private Color _topColor = Color.FromArgb(223, 205, 117);
        private Color _bottomColor = Color.FromArgb(207, 188, 99);
        private BufferedGraphics _bufGFX;
        private BufferedGraphicsContext _bufContext = BufferedGraphicsManager.Current;
        private GraphicsPath _GP;
        private LinearGradientBrush _LGB, _LGBDesat;
        private SolidBrush _textBrush = new SolidBrush(Color.FromArgb(83, 55, 19));
        private Pen _shinePen = new Pen(Color.FromArgb(236, 225, 174), 2f);
        private bool _isMouseOver;

        public KATButton()
        {
            this.SetStyle(ControlStyles.OptimizedDoubleBuffer | ControlStyles.UserPaint
                | ControlStyles.AllPaintingInWmPaint | ControlStyles.ResizeRedraw, true);

            _shinePen.EndCap = _shinePen.StartCap = LineCap.Round;
            this.Font = new Font("Tahoma", 9f);
            UpdateGraphics();
        }

        private void UpdateGraphics()
        {
            if (this.Width > 0 && this.Height > 0)
            {
                _GP = RoundedRectangle.Create(0, 0, this.Width - 2, this.Height - 2, _cornerRadius);
                this.Region = new Region(_GP);

                _bufContext.MaximumBuffer = new Size(this.Width + 1, this.Height + 1);
                _bufGFX = _bufContext.Allocate(this.CreateGraphics(), this.ClientRectangle);
                Rectangle rect = new Rectangle(Point.Empty, this.Size);
                _LGB = new LinearGradientBrush(rect, _topColor, _bottomColor, 90f);

                Color topDarkColor = ChangeColorBrightness(_topColor, -20);
                Color bottomDarkColor = ChangeColorBrightness(_bottomColor, -20);
                _LGBDesat = new LinearGradientBrush(rect, topDarkColor, bottomDarkColor, 90f);
            }
        }

        protected override void OnSizeChanged(EventArgs e)
        {
            base.OnSizeChanged(e);
            UpdateGraphics();
        }

        private static Color ChangeColorBrightness(Color color, int factor)
        {
            int R = (color.R + factor > 255) ? 255 : color.R + factor;
            int G = (color.G + factor > 255) ? 255 : color.G + factor;
            int B = (color.B + factor > 255) ? 255 : color.B + factor;
            if (R < 0) R = 0;
            if (B < 0) B = 0;
            if (G < 0) G = 0;

            return Color.FromArgb(R, G, B);
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            _bufGFX.Graphics.Clear(this.BackColor);

            if (_isMouseOver)
            {
                _bufGFX.Graphics.FillPath(_LGBDesat, _GP);
            }
            else
            {
                _bufGFX.Graphics.FillPath(_LGB, _GP);
            }

            // Draw shine
            _bufGFX.Graphics.SmoothingMode = SmoothingMode.AntiAlias;
            _bufGFX.Graphics.PixelOffsetMode = PixelOffsetMode.HighQuality;
            _bufGFX.Graphics.DrawLine(_shinePen, new Point(3, 2), new Point(this.Width - 6, 2));
            _bufGFX.Graphics.PixelOffsetMode = PixelOffsetMode.Default;
            _bufGFX.Graphics.SmoothingMode = SmoothingMode.Default;

            // Draw Image 
            if (this.Enabled && this.BackgroundImage != null)
            {
                if (AutoSizeImage)
                {
                    Rectangle rect = this.ClientRectangle;
                    rect.Inflate(-10, -10);
                    _bufGFX.Graphics.DrawImage(this.BackgroundImage, rect);
                }
                else
                {
                    int x = this.Width / 2 - this.BackgroundImage.Width / 2;
                    int y = this.Height / 2 - this.BackgroundImage.Height / 2;
                    _bufGFX.Graphics.DrawImage(this.BackgroundImage, new Point(x, y));
                }
            } // Draw disabled image
            else if (!this.Enabled && DisabledBackgroundImage != null)
            {
                if (AutoSizeImage)
                {
                    Rectangle rect = this.ClientRectangle;
                    rect.Inflate(-10, -10);
                    _bufGFX.Graphics.DrawImage(this.DisabledBackgroundImage, rect);
                }
                else
                {
                    int x = this.Width / 2 - this.DisabledBackgroundImage.Width / 2;
                    int y = this.Height / 2 - this.DisabledBackgroundImage.Height / 2;
                    _bufGFX.Graphics.DrawImage(this.DisabledBackgroundImage, new Point(x, y));
                }
            }

            // Draw string
            SizeF stringSize = _bufGFX.Graphics.MeasureString(this.Text, this.Font);
            PointF pos = new PointF(this.Width / 2 - stringSize.Width / 2, this.Height / 2 - stringSize.Height / 2);

            if (this.Enabled)
            {
                _bufGFX.Graphics.DrawString(this.Text, this.Font, _textBrush, pos);
            }
            else
            {
                // Draw gray text when disabled
                _bufGFX.Graphics.DrawString(this.Text, this.Font, Brushes.Gray, pos);
            }

            _bufGFX.Render(e.Graphics);
        }

        protected override void OnFontChanged(EventArgs e)
        {
            base.OnFontChanged(e);
            this.Invalidate();
        }

        protected override void OnTextChanged(EventArgs e)
        {
            base.OnTextChanged(e);
            this.Invalidate();
        }

        protected override void OnMouseEnter(EventArgs e)
        {
            base.OnMouseEnter(e);
            _isMouseOver = true;

            if (this.Enabled)
            {
                this.Cursor = Cursors.Hand;
            }

            this.Invalidate();
        }

        protected override void OnMouseLeave(EventArgs e)
        {
            base.OnMouseLeave(e);
            _isMouseOver = false;
            this.Cursor = Cursors.Default;
            this.Invalidate();
        }

        [Category("Appearance")]
        public Image DisabledBackgroundImage
        {
            get { return _disabledBackgroundImage; }
            set 
            { 
                _disabledBackgroundImage = value;
                this.Invalidate();
            }
        }

        /// <summary>
        /// The roundness of a button
        /// </summary>
        public int CornerRadius
        {
            get { return _cornerRadius; }
            set 
            {
                _cornerRadius = value;
                UpdateGraphics();
                this.Invalidate();
            }
        }

        private abstract class RoundedRectangle
        {
            public enum RectangleCorners
            {
                None = 0, TopLeft = 1, TopRight = 2, BottomLeft = 4, BottomRight = 8,
                All = TopLeft | TopRight | BottomLeft | BottomRight
            }

            public static GraphicsPath Create(int x, int y, int width, int height,
                                              int radius, RectangleCorners corners)
            {
                int xw = x + width;
                int yh = y + height;
                int xwr = xw - radius;
                int yhr = yh - radius;
                int xr = x + radius;
                int yr = y + radius;
                int r2 = radius * 2;
                int xwr2 = xw - r2;
                int yhr2 = yh - r2;

                GraphicsPath p = new GraphicsPath();
                p.StartFigure();

                //Top Left Corner
                if ((RectangleCorners.TopLeft & corners) == RectangleCorners.TopLeft)
                {
                    p.AddArc(x, y, r2, r2, 180, 90);
                }
                else
                {
                    p.AddLine(x, yr, x, y);
                    p.AddLine(x, y, xr, y);
                }

                //Top Edge
                p.AddLine(xr, y, xwr, y);

                //Top Right Corner
                if ((RectangleCorners.TopRight & corners) == RectangleCorners.TopRight)
                {
                    p.AddArc(xwr2, y, r2, r2, 270, 90);
                }
                else
                {
                    p.AddLine(xwr, y, xw, y);
                    p.AddLine(xw, y, xw, yr);
                }

                //Right Edge
                p.AddLine(xw, yr, xw, yhr);

                //Bottom Right Corner
                if ((RectangleCorners.BottomRight & corners) == RectangleCorners.BottomRight)
                {
                    p.AddArc(xwr2, yhr2, r2, r2, 0, 90);
                }
                else
                {
                    p.AddLine(xw, yhr, xw, yh);
                    p.AddLine(xw, yh, xwr, yh);
                }

                //Bottom Edge
                p.AddLine(xwr, yh, xr, yh);

                //Bottom Left Corner
                if ((RectangleCorners.BottomLeft & corners) == RectangleCorners.BottomLeft)
                {
                    p.AddArc(x, yhr2, r2, r2, 90, 90);
                }
                else
                {
                    p.AddLine(xr, yh, x, yh);
                    p.AddLine(x, yh, x, yhr);
                }

                //Left Edge
                p.AddLine(x, yhr, x, yr);

                p.CloseFigure();
                return p;
            }

            public static GraphicsPath Create(Rectangle rect, int radius, RectangleCorners c)
            { return Create(rect.X, rect.Y, rect.Width, rect.Height, radius, c); }

            public static GraphicsPath Create(int x, int y, int width, int height, int radius)
            { return Create(x, y, width, height, radius, RectangleCorners.All); }

            public static GraphicsPath Create(Rectangle rect, int radius)
            { return Create(rect.X, rect.Y, rect.Width, rect.Height, radius); }

            public static GraphicsPath Create(int x, int y, int width, int height)
            { return Create(x, y, width, height, 5); }

            public static GraphicsPath Create(Rectangle rect)
            { return Create(rect.X, rect.Y, rect.Width, rect.Height); }
        }
    }
}
