﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.IO.Compression;
using System.Linq;
using System.Net;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;
using BSFControls.ExtendedControls;
using KAT_Browser.Properties;

namespace KAT_Browser
{
    public partial class MainForm : Form
    {
        private PageManager _pageManager = new PageManager();

        public MainForm()
        {
            InitializeComponent();
            LoadSettings();
            SetDefaultPreviewPage();
            _pageManager.HasPreviousValueChanged += _pageManager_HasPreviousValueChanged;
            _pageManager.HasNextValueChanged += _pageManager_HasNextValueChanged;
            _pageManager.CurrentFinishedLoading += _pageManager_CurrentFinishedLoading;
        }

        /// <summary>
        /// Makes the webbrowser black for now I may add details later
        /// </summary>
        private void SetDefaultPreviewPage()
        {
            const string content = @"<html style=""background-color:black""><body></body></html>";
            webBrowser.Document.Write(content);
        }

        private void LoadSettings()
        {
            // Set  torrent download dest if not set
            if (Settings.Default.TorrentDest.Length == 0)
            {
                string path = Application.StartupPath + "\\Torrents";
                Settings.Default.TorrentDest = path;
            }

            txtSearchTerm.Text = Settings.Default.SearchTerm;
            this.Bounds = Settings.Default.Bounds;

            if (Settings.Default.Bounds == Rectangle.Empty)
            {
                this.CenterToScreen();
            }

            splitContainer.SplitterDistance = Settings.Default.SplitterDistance;
        }

        private void TogglePreview()
        {
            if (splitContainer.Panel2Collapsed)
            {
                btnTogglePreview.Text = ">";
                splitContainer.Panel2Collapsed = false;
            }
            else
            {
                btnTogglePreview.Text = "<";
                splitContainer.Panel2Collapsed = true;
            }
        }

        private void InitializeSearch()
        {
            SetControlStates(false);
            _pageManager.SetCurrentPage(txtSearchTerm.Text, 1);
        }

        /// <summary>
        /// Pass in false if you are loading a page
        /// </summary>
        private void SetControlStates(bool state)
        {
            btnSearch.Enabled = state;
            pnlTorrentLister.Visible = state;
            activityIndicator.Visible = !state;

            if (!state) activityIndicator.BringToFront();
        }

        private void UpdateGUI()
        {
            Stack<TorrentDisplay> torrentDisplays = new Stack<TorrentDisplay>();

            for (int i = 0; i < _pageManager.Current.Torrents.Length; i++)
            {
                Torrent current = _pageManager.Current.Torrents[i];
                TorrentDisplay display = new TorrentDisplay(current);

                if (i % 2 == 0) // Alternate colors
                {
                    display.BackColor = Color.FromArgb(244, 241, 231);
                }
                else
                {
                    display.BackColor = Color.FromArgb(255, 255, 255);
                }

                display.MouseLeaveColor = display.BackColor;
                display.Click += display_Click;
                display.Dock = DockStyle.Top;
                torrentDisplays.Push(display);
            }

            pnlTorrentLister.SuspendLayout();
            pnlTorrentLister.Controls.Clear();
            pnlTorrentLister.Controls.AddRange(torrentDisplays.ToArray());

            pnlTorrentLister.ResumeLayout();
            string strPgNum = _pageManager.Current.PageNumber.ToString();
            this.Text = Application.ProductName + " - Page " + strPgNum;
        }

        #region Eventing
        private void btnTogglePreview_Click(object sender, EventArgs e)
        {
            TogglePreview();
        }

        private void btnSettings_Click(object sender, EventArgs e)
        {
            using (SettingsForm form = new SettingsForm())
            {
                form.ShowDialog();
            }
        }

        private void txtSearchTerm_TextChanged(object sender, EventArgs e)
        {
            btnSearch.Enabled = txtSearchTerm.TextLength > 0;
        }

        protected override void OnKeyDown(KeyEventArgs e)
        {
            base.OnKeyDown(e);

            if (e.KeyCode == Keys.Enter && txtSearchTerm.TextLength > 0 && !activityIndicator.Visible)
            {
                e.SuppressKeyPress = true;
                InitializeSearch();
            }
        }

        protected override void OnClosing(CancelEventArgs e)
        {
            // Save settings
            Settings.Default.Bounds = this.Bounds;
            Settings.Default.SplitterDistance = splitContainer.SplitterDistance;
            Settings.Default.SearchTerm = txtSearchTerm.Text;
            Settings.Default.Save();
            base.OnClosing(e);
        }

        private void display_Click(object sender, EventArgs e)
        {
            PreviewRetriever previewWorker = new PreviewRetriever();
            previewWorker.RunWorkerCompleted += previewWorker_RunWorkerCompleted;
            TorrentDisplay current = (TorrentDisplay)sender;
            previewWorker.RunWorkerAsync(current.Torrent.Name);
        }

        private void previewWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            if (e.Error == null)
            {
                webBrowser.Navigate(e.Result.ToString());

                if (splitContainer.Panel2Collapsed)
                {
                    TogglePreview();
                }
            }
            else
            {
                MessageBox.Show(e.Error.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btnLink_Click(object sender, EventArgs e)
        {
            try
            {
                Process.Start("http://kat.ph");
            }
            catch (Win32Exception ex)
            {
                MessageBox.Show(ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void _pageManager_CurrentFinishedLoading(object sender, EventArgs e)
        {
            UpdateGUI();
            SetControlStates(true);

        }

        private void _pageManager_HasPreviousValueChanged(object sender, EventArgs e)
        {
            btnPrevious.Enabled = _pageManager.HasPrevious;
        }

        private void _pageManager_HasNextValueChanged(object sender, EventArgs e)
        {
            btnNext.Enabled = _pageManager.HasNext;
        }

        private void btnPrevious_Click(object sender, EventArgs e)
        {
            _pageManager.GoToPreviousPage();
            UpdateGUI();
        }

        private void btnNext_Click(object sender, EventArgs e)
        {
            _pageManager.GoToNextPage();
            UpdateGUI();
        }

        private void btnSearch_Click(object sender, EventArgs e)
        {
            InitializeSearch();
        }
        #endregion
    }
}
