﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.IO;
using System.IO.Compression;
using System.Linq;
using System.Net;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace KAT_Browser
{
    class PageManager
    {
        private int _maxPages = 1;
        private static readonly string _SEARCH_FORMAT = @"http://kat.ph/usearch/{0}/{1}/";
        private bool _hasPrevious, _hasNext;
        private KATSearchPage _nextPage = new KATSearchPage();
        private KATSearchPage _previousPage = new KATSearchPage();
        private KATSearchPage _currentPage = new KATSearchPage();
        private WebPageDownloader _nextDownloader = new WebPageDownloader();
        private WebPageDownloader _previousDownloader = new WebPageDownloader();
        private WebPageDownloader _currentDownloader = new WebPageDownloader();
        public event EventHandler HasNextValueChanged, HasPreviousValueChanged, CurrentFinishedLoading;

        public PageManager()
        {
            _nextDownloader.RunWorkerCompleted += _nextDownloader_RunWorkerCompleted;
            _previousDownloader.RunWorkerCompleted += _previousDownloader_RunWorkerCompleted;
            _currentDownloader.RunWorkerCompleted += _currentDownloader_RunWorkerCompleted;
        }

        #region Eventing
        private void _currentDownloader_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                if (e.Error is WebException)
                {
                    MessageBox.Show("No Results found", Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                }
                else
                {
                    const string MSG = "Failed to download current page. Try again later";
                    MessageBox.Show(MSG, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }

                OnCurrentFinishedLoading();
                return;
            }

            if (e.Result != null)
            {
                _currentPage.LoadPage(e.Result.ToString());
                SetMaxPages();

                // Set next page
                if (_currentPage.NextPageLink.Length > 0)
                {
                    string URL = String.Format(_SEARCH_FORMAT, _currentPage.SearchTerm, _currentPage.PageNumber + 1);
                    _nextDownloader.RunWorkerAsync(URL);
                }

                // Set previous page
                if (_currentPage.PreviousPageLink.Length > 0)
                {
                    string URL = String.Format(_SEARCH_FORMAT, _currentPage.SearchTerm, _currentPage.PageNumber - 1);
                    _previousDownloader.RunWorkerAsync(URL);
                }

                OnCurrentFinishedLoading();
            }
        }

        private void _previousDownloader_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            if (e.Error == null)
            {
                if (e.Result != null)
                {
                    string result = e.Result.ToString();

                    if (result.Length > 0)
                    {
                        _previousPage.LoadPage(result);
                        HasPrevious = true;
                    }
                    else
                    {
                        Debug.WriteLine("Previous page downloaded, but content is empty");
                        HasPrevious = false;
                    }
                }
                else Debug.WriteLine("Previous page downloaded, but content is null");
            }
            else
            {
                const string MSG = "Failed to download previous current page. You will not be able to go back";
                MessageBox.Show(MSG, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
            }
        }

        private void _nextDownloader_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            if (e.Error == null)
            {
                if (e.Result != null)
                {
                    string result = e.Result.ToString();

                    if (result.Length > 0)
                    {
                        _nextPage.LoadPage(result);
                        HasNext = true;
                    }
                    else
                    {
                        Debug.WriteLine("Next page downloaded, but content is empty");
                        HasNext = false;
                    }
                }
            }
            else
            {
                const string MSG = "Failed to download next page. Try again later";
                MessageBox.Show(MSG, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
            }
        }

        private void OnHasNextValueChanged()
        {
            if (HasNextValueChanged != null)
            {
                HasNextValueChanged(this, EventArgs.Empty);
            }
        }

        private void OnHasPreviousValueChanged()
        {
            if (HasPreviousValueChanged != null)
            {
                HasPreviousValueChanged(this, EventArgs.Empty);
            }
        }

        private void OnCurrentFinishedLoading()
        {
            if (CurrentFinishedLoading != null)
            {
                CurrentFinishedLoading(this, EventArgs.Empty);
            }
        }
        #endregion

        private void SetMaxPages()
        {
            const string PATTERN = @"href=""/usearch/\w+/(?<Page>\d+)/""";
            MatchCollection MC = Regex.Matches(_currentPage.Content, PATTERN);
            int highestNum = 1;

            foreach (Match match in MC)
            {
                string value = match.Groups["Page"].Value;
                int num = int.Parse(value);
                if (num > highestNum) highestNum = num;
            }

            _maxPages = highestNum;
        }

        public void SetCurrentPage(string searchTerm, int pageNum)
        {
            // Set current
            searchTerm = searchTerm.Replace(" ", "%20");
            string URL = String.Format(_SEARCH_FORMAT, searchTerm, pageNum);
            _currentDownloader.RunWorkerAsync(URL);
        }

        public void GoToNextPage()
        {
            _currentPage.LoadPage(_nextPage.Content);
            UpdateNearPages();
        }

        public void GoToPreviousPage()
        {
            _currentPage.LoadPage(_previousPage.Content);
            UpdateNearPages();
        }

        private void UpdateNearPages()
        {
            // Set next page
            if (_currentPage.NextPageLink.Length > 0 && !_nextDownloader.IsBusy)
            {
                HasNext = false;
                _nextDownloader.RunWorkerAsync(_currentPage.NextPageLink);
            }

            // Set previous page
            if (_currentPage.PreviousPageLink.Length > 0 && !_previousDownloader.IsBusy)
            {
                HasPrevious = false;
                _previousDownloader.RunWorkerAsync(_currentPage.PreviousPageLink);
            }
        }

        /// <summary>
        /// Gets the currently loaded KAT search page
        /// </summary>
        public KATSearchPage Current
        {
            get { return _currentPage; }
        }

        /// <summary>
        /// Determines whether there is a next page
        /// </summary>
        public bool HasNext
        {
            get { return _hasNext; }
            set
            {
                if (_currentPage.PageNumber == MaxPages) value = false;
                Debug.WriteLine("Current Page num:" + _currentPage.PageNumber + " MaxPage:" + MaxPages);
                bool changed = (value != _hasNext);
                _hasNext = value;
                OnHasNextValueChanged();
            }
        }

        public bool HasPrevious
        {
            get { return _hasPrevious; }
            set
            {
                // Do not allow to set to true if we are still on page 1
                // we don not have a previous page on one
                if (_currentPage.PageNumber == 1) value = false;
                bool changed = (value != _hasPrevious);
                _hasPrevious = value;
                OnHasPreviousValueChanged();
            }
        }

        /// <summary>
        /// Gets the maximum amount of pages a user could possibly navigate for the current search
        /// </summary>
        private int MaxPages
        {
            get { return _maxPages; }
        }
    }
}
